@@ -71,7 +71,12 @@ Vue.component( 'foreign-key-field', {
             valueField: this.$props.valueField || schema['x-id-field'] || 'id'
         };
 
-        if ( !data.displayField ) {
+        if ( data.displayField ) {
+            if ( typeof data.displayField == 'object' && data.displayField.template ) {
+                data.displayTemplate = data.displayField.template;
+            }
+        }
+        else {
             var schemaCols = schema['x-list-columns'];
             if ( schemaCols && typeof schemaCols[0] == 'object' ) {
                 data.displayTemplate = schemaCols[0].template;
